/*
 *  xtime.c -  create a "file" in /proc with xtime global kernel value
 *
 * Compile this with the makefile (type make)
 */

/* Standard headers for LKMs */
#include <linux/module.h>  
#include <linux/config.h>  
#include <linux/init.h>  

#include <linux/proc_fs.h>  /* Necessary because we use the proc fs */
/* #include <linux/tty.h>       console_print() interface */

MODULE_LICENSE("GPL");

struct proc_dir_entry *Our_Proc_File;

/* Put data into the proc fs file.
 * 
 * Usage and Return Value
 * ======================
 * A return value of zero means you have no further
 * information at this time (end of file). A negative
 * return value is an error condition.
 */

ssize_t procfile_read(char *buffer,
         char **buffer_location,
         off_t offset, int buffer_length, int *eof, void *data)
{
   printk(KERN_INFO "inside /proc/xtime : procfile_read\n");
   int len = 0;      /* The number of bytes actually used */

   if (offset > 0) {
      printk(KERN_INFO "offset %d : /proc/xtime : procfile_read, \
             wrote %d Bytes\n", (int)(offset), len);
      *eof = 1;
      return len;
   }

   /* 
    * Fill the buffer and get its length 
    */
   len = sprintf(buffer, "%d %d\n", (int)xtime.tv_sec,
                                    (int)xtime.tv_nsec);
   /* 
    * Return the length 
    */
   printk(KERN_INFO
          "leaving /proc/xtime : procfile_read, wrote %d Bytes\n", len);
   return len;
}

static int __init init_xtime(void)
{
   int rv = 0;
   Our_Proc_File = create_proc_entry("xtime", 0644, NULL);
   Our_Proc_File->read_proc = procfile_read;
   Our_Proc_File->owner = THIS_MODULE;
   Our_Proc_File->mode = S_IFREG | S_IRUGO;
   Our_Proc_File->uid = 0;
   Our_Proc_File->gid = 0;
   Our_Proc_File->size = 37;

   printk(KERN_INFO "Trying to create /proc/xtime:\n");

   if (Our_Proc_File == NULL) {
      rv = -ENOMEM;
      remove_proc_entry("xtime", &proc_root);
      printk(KERN_INFO "Error: Could not initialize /proc/xtime\n");
   } else {
      printk(KERN_INFO "Success!\n");
   }

   return rv;
}

static void __exit cleanup_xtime(void)
{
   remove_proc_entry("xtime", &proc_root);
   printk(KERN_INFO "/proc/xtime removed\n");
}

module_init(init_xtime);
module_exit(cleanup_xtime);
